#coding=utf-8

import sys
import os
import os.path
import argparse
import urllib.request as UR
import urllib.error as UE
import hashlib

import lonote.translate
import lonote.server
import lonote.info

def exec_dir():
    '''取得執行檔所在資料夾'''
    return os.path.abspath(os.path.dirname(sys.argv[0]))

def config_path():
    '''取得設定路徑'''
    def userdir():
        '''取得當前的使用者資料夾'''
        # 環境變數 HOME 用在 Linux 系統，USERPROFILE 用在 Windows 系統
        return os.getenv('HOME') or os.getenv('USERPROFILE')
    def gen_path(cfgpath):
        '''如果有需要，預先產生空路徑'''
        lonotecfg_dir = os.path.join(cfgpath, 'lonote')
        try: os.makedirs(lonotecfg_dir, exist_ok = True)
        except: pass

    cfgpath = os.path.join(exec_dir(), 'config')
    if os.path.isdir(cfgpath): # 使用資料夾路徑
        return cfgpath
    else: # 使用程式設定路徑
        userdir = userdir()
        cfgpath = os.path.join(userdir, '.config')
        gen_path(cfgpath)
        return cfgpath

def server_path():
    '''取得 server 路徑'''
    serpath = os.path.join(exec_dir(), 'server')
    if os.path.isdir(serpath): # 使用資料夾路徑
        return serpath
    else: # 使用程式設定路徑
        serpath = os.path.join(
                os.path.dirname(exec_dir()),
                'share',
                'lonote',
                'server')
        return serpath

def translate_ready():
    '''讓 locale 路徑準備好並完成 _() 的初期安裝'''
    src_locale_dir = os.path.join(exec_dir(), 'locale')
    lonote.translate.set_locale_dir(src_locale_dir)
    # lonote.translate.install()
    lonote.translate.get_trans(None).install()

def parse_argv():
    '''回傳分析後的 argv'''
    description = _('''LoNote is a personal note-taking software base on modern browser an python 3.''')

    parser = argparse.ArgumentParser(
            description = description,
            #formatter_class = argparse.ArgumentDefaultsHelpFormatter
            )
    parser.add_argument('-b', '--browser',
            default = '',
            help = _('A browser you want to open. (linux only)'))
    parser.add_argument('-d', '--debug',
            dest = 'debug',
            action = 'store_true',
            help = _('Use debug mode'))
    parser.add_argument('-n','--no-browser',
            dest = 'nobrowser',
            action = 'store_true',
            help = _('Do not auto-open any browser.'))
    parser.add_argument('-p', '--port',
            dest = 'port',
            type = int,
            default = 10204,
            help = _('Set the server port.'))
    parser.add_argument('--remote',
            dest = 'remote',
            default = False,
            action = 'store_true',
            help = _('Let other computer can connect to this server. (be careful! LoNote auth system not very secure yet. So any skilled cracker may touch your data!)'))
    parser.add_argument('--version',
            dest = 'version',
            action = 'store_true',
            help = _('Show version and exit.'))
    parser.add_argument('--stop',
            dest = 'stop',
            default = False,
            action = 'store_true',
            help = _('Stop server on target port.'))
    parser.add_argument('--restart',
            dest = 'restart',
            default = False,
            action = 'store_true',
            help = _('Restart server on target port.'))
    # parser.add_argument('--dir-book',
    #         dest = 'booksdir',
    #         default = BOOKS_DIR,
    #         help = _('All notebooks must be placed in here.'))
    # parser.add_argument('--dir-index',
    #         dest = 'indexdir',
    #         default = INDEX_DIR,
    #         help = _('All search index will be stored in here.'))
    # parser.add_argument('--convert',
    #         dest = 'convert',
    #         nargs = 1,
    #         help = _('Convert a old notebook (LoNote version >= 1.8 and <=1.8.7) To a LoNote 2.X format, and exit.'))
    # parser.add_argument('--convert-keep-history',
    #         dest = 'convert_keep_history',
    #         action = 'store_true',
    #         help = _('Keep all history when --convert be used.'))

    args = parser.parse_args()
    return args

def stop_server(port):
    '''測試並關閉目標 port 上正在運作的 LoNote 伺服器'''
    # peocess auth_code
    lonote.server.init_paths(config_path(), server_path())
    f_code = lonote.server.CONFIG.get('AUTHCODE', 'full_access_code')
    f_code = hashlib.md5(f_code.encode('utf-8')).hexdigest()
    opener = UR.build_opener()
    opener.addheaders.append( ('Cookie', 'auth_code={}'.format(f_code)))

    test_target = 'http://localhost:{}'.format(port)
    shutdown_target = 'http://localhost:{}/func/shutdown'.format(port)
    try:
        opener.open(test_target)
        try:
            opener.open(shutdown_target, data = b'')
        except UE.HTTPError:
            print('Shutdown failed, Server already run on {} port.'.format(port))
        except Exception:
            print('Shutdown Success on {} port.'.format(port))
        else:
            print('Shutdown failed, Server already run on {} port.'.format(port))
    except UE.URLError:
        print('No LoNote Server run on {} port.'.format(port))

def process_argv(argv):
    '''處理由 parse_argv 準備好的資料'''
    # 進行不需啟動伺服器的處理
    # version
    if argv.version:
        print(lonote.info.version)
        sys.exit(0)

    # stop & restart
    if argv.stop or argv.restart:
        stop_server(argv.port)
        if not argv.restart:
            sys.exit(0)

    # 啟動伺服器
    cfg_dir = config_path()
    ser_dir = server_path()
    if argv.nobrowser:
        lonote.server.start_server(
                cfgdir = cfg_dir, serdir = ser_dir,
                remote = argv.remote, port = argv.port,
                debug = argv.debug, browser = None )
    else:
        lonote.server.start_server(
                cfgdir = cfg_dir, serdir = ser_dir,
                remote = argv.remote, port = argv.port,
                debug = argv.debug, browser = argv.browser )

def run():
    '''執行'''
    # prepare default data and url
    translate_ready()
    # Parse argv
    argv = parse_argv()
    process_argv(argv)
